@@ -1156,6 +1156,10 @@ ChertTable::readahead_key(const string &key) const
     if (level == 0)
 	RETURN(false);
 
+    // An overlong key cannot be found.
+    if (key.size() > CHERT_BTREE_MAX_KEY_LEN)
+	RETURN(true);
+
     form_key(key);
     Key ktkey = kt.key();
 
