@@ -1446,6 +1446,10 @@ GlassTable::readahead_key(const string &key) const
     if (level == 0)
 	RETURN(false);
 
+    // An overlong key cannot be found.
+    if (key.size() > GLASS_BTREE_MAX_KEY_LEN)
+	RETURN(true);
+
     form_key(key);
 
     // We'll only readahead the first level, since descending the B-tree would
