@@ -1,4 +1,4 @@
-/** @file expanddecider.h
+/** @file
  * @brief Allow rejection of terms during ESet generation.
  */
 /* Copyright (C) 2007,2011,2013,2014,2015,2016 Olly Betts
@@ -22,7 +22,7 @@
 #define XAPIAN_INCLUDED_EXPANDDECIDER_H
 
 #if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
-# error "Never use <xapian/expanddecider.h> directly; include <xapian.h> instead."
+# error Never use <xapian/expanddecider.h> directly; include <xapian.h> instead.
 #endif
 
 #include <set>
