@@ -260,11 +260,19 @@ TermGenerator::Internal::index_text(Utf8Iterator itor, termcount wdf_inc,
     }
 
     parse_terms(itor, cjk_ngram, with_positions,
-	[=](const string & term, bool positional, const Utf8Iterator &) {
+	[=
+#if __cplusplus >= 201907L
+// C++20 no longer supports implicit `this` in lambdas but older C++ versions
+// don't allow `this` here.
+	, this
+#endif
+	](const string & term, bool positional, const Utf8Iterator &) {
 	    if (term.size() > max_word_length) return true;
 
-	    if (current_stop_mode == TermGenerator::STOP_ALL && (*stopper)(term))
+	    if (current_stop_mode == TermGenerator::STOP_ALL &&
+		(*stopper)(term)) {
 		return true;
+	    }
 
 	    if (strategy == TermGenerator::STEM_SOME ||
 		strategy == TermGenerator::STEM_NONE ||
